==> PyNFe-1.0.22.tar.gz: PyNFe-1.0.22/pynfe/xml_sped/certificado.py <==
# -*- coding: utf-8 -*-

from pynfe.xml_sped.base import XMLNFe, ABERTURA
from pynfe.xml_sped.assinatura import Signature
from datetime import datetime
from OpenSSL import crypto
from lxml import etree
import signxml
import os


DIRNAME = os.path.dirname(__file__)


class Certificado(object):
    def __init__(self):
        self.arquivo = ''
        self.senha = ''
        self.chave = ''
        self.certificado = ''
        self.emissor = {}
        self.proprietario = {}
        self.data_inicio_validade = None
        self.data_fim_validade = None
        self._doc_xml = None
        self.cert_str=''
        self.key_str=''


    def _set_chave(self, chave):
        self._chave = chave

    def _get_chave(self):
        try:
            if self._chave:
                return self._chave
            else:
                raise AttributeError("'chave' precisa ser regenerada")
        except AttributeError as e:
            if self.arquivo:  # arquivo disponível
                self.prepara_certificado_arquivo_pfx()
                return self._chave  # agora já disponível
            else:
                return ''

    chave = property(_get_chave, _set_chave)

    def _set_certificado(self, certificado):
        self._certificado = certificado

    def _get_certificado(self):
        try:
            if self._certificado:
                return self._certificado
            else:
                raise AttributeError("'certificado' precisa ser regenerado")
        except AttributeError as e:
            if self.arquivo:  # arquivo disponível
                self.prepara_certificado_arquivo_pfx()
                return self._certificado  # agora já disponível
            else:
                return ''

    certificado = property(_get_certificado, _set_certificado)

    @property
    def proprietario_nome(self):
        if 'CN' in self.proprietario:
            #
            # Alguns certrificados não têm o CNPJ na propriedade CN, somente o
            # nome do proprietário
            #
            if ':' in self.proprietario['CN']:
                return self.proprietario['CN'].rsplit(':', 1)[0]
            else:
                return self.proprietario['CN']
        else: # chave CN ainda não disponível
            try:
                self.prepara_certificado_arquivo_pfx()
                return self.proprietario['CN'].rsplit(':', 1)[0]
            except IOError as e:  # arquivo do certificado não disponível
                return ''

    @property
    def proprietario_cnpj(self):
        if 'CN' in self.proprietario:
            #
            # Alguns certrificados não têm o CNPJ na propriedade CN, somente o
            # nome do proprietário
            #
            if ':' in self.proprietario['CN']:
                return self.proprietario['CN'].rsplit(':', 1)[1]
            else:
                return ''
        else:  # chave CN ainda não disponível
            try:
                self.prepara_certificado_arquivo_pfx()
                return self.proprietario['CN'].rsplit(':', 1)[1]
            except IOError as e:  # arquivo do certificado não disponível
                return ''

    def prepara_certificado_arquivo_pfx(self):

        self.chave=self.key_str
        self.prepara_certificado_txt(self.cert_str)

    def prepara_certificado_arquivo_pem(self):
        self.prepara_certificado_txt(self.cert_str)

    def prepara_certificado_txt(self, cert_txt):
        #
        # Para dar certo a leitura pelo xmlsec, temos que separar o certificado
        # em linhas de 64 caracteres de extensão...
        #
        cert_txt = cert_txt.replace('\n', '')
        cert_txt = cert_txt.replace('-----BEGIN CERTIFICATE-----', '')
        cert_txt = cert_txt.replace('-----END CERTIFICATE-----', '')
        linhas_certificado = ['-----BEGIN CERTIFICATE-----\n']
        for i in range(0, len(cert_txt), 64):
            linhas_certificado.append(cert_txt[i:i+64] + '\n')
        linhas_certificado.append('-----END CERTIFICATE-----\n')
        self.certificado = ''.join(linhas_certificado)
        cert_openssl = crypto.load_certificate(crypto.FILETYPE_PEM, self.certificado)
        self.emissor = dict(cert_openssl.get_issuer().get_components())
        self.proprietario = dict(cert_openssl.get_subject().get_components())
        self.data_inicio_validade = datetime.strptime(
            cert_openssl.get_notBefore().decode('utf-8'), '%Y%m%d%H%M%SZ')
        self.data_fim_validade = datetime.strptime(
            cert_openssl.get_notAfter().decode('utf-8'), '%Y%m%d%H%M%SZ')

    def assina_xmlnfe(self, doc):
        if not isinstance(doc, XMLNFe):
            raise ValueError('O documento nao e do tipo esperado: XMLNFe')

        # Realiza a assinatura
        xml = self.assina_xml(doc.xml)

        # Devolve os valores para a instância doc
        doc.Signature.xml = xml

    def assina_arquivo(self, doc):
        xml = open(doc, 'r').read()
        xml = self.assina_xml(xml)
        return xml

    def _prepara_doc_xml(self, xml):
        #
        # Determina o tipo de arquivo que vai ser assinado, procurando
        # pela tag correspondente
        #
        if 'infNFe' in xml:
            doctype = '<!DOCTYPE NFe [<!ATTLIST infNFe Id ID #IMPLIED>]>'
        elif 'infCanc' in xml:
            doctype = '<!DOCTYPE cancNFe [<!ATTLIST infCanc Id ID #IMPLIED>]>'
        elif 'infEvento' in xml:
            doctype = '<!DOCTYPE envEvento [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        elif 'infInut' in xml:
            doctype = '<!DOCTYPE inutNFe [<!ATTLIST infInut Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        #
        # Importantíssimo colocar o encode, pois do contário não é possível
        # assinar caso o xml tenha letras acentuadas
        #
        xml = xml.replace(ABERTURA, '')
        xml = ABERTURA + xml
        xml = xml.replace(ABERTURA, ABERTURA + doctype)

        #
        # Remove todos os \n
        #
        xml = xml.replace('\n', '')

        return xml

    def _finaliza_xml(self, xml):
        #
        # Determina o tipo de arquivo que vai ser assinado, procurando
        # pela tag correspondente
        #
        if 'infNFe' in xml:
            doctype = '<!DOCTYPE NFe [<!ATTLIST infNFe Id ID #IMPLIED>]>'
        elif 'infCanc' in xml:
            doctype = '<!DOCTYPE cancNFe [<!ATTLIST infCanc Id ID #IMPLIED>]>'
        elif 'infEvento' in xml:
            doctype = '<!DOCTYPE envEvento [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        elif 'infInut' in xml:
            doctype = '<!DOCTYPE inutNFe [<!ATTLIST infInut Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        #
        # Remove o doctype e os \n acrescentados pela libxml2
        #
        xml = xml.replace('\n', '')
        xml = xml.replace(ABERTURA + doctype, ABERTURA)

        return xml

    def assina_xml(self, xml, metodo='sha1'):
        xml = self._prepara_doc_xml(xml)
        assinatura = Signature()
        assinatura.xml = xml

        # Tiramos a tag de assinatura, o signxml vai colocar de novo
        if '<Signature ' in xml:
            partes = xml.split('<Signature ')
            xml_sem_assinatura = ''.join(partes[:-1])
            partes = partes[-1].split('</Signature>')
            xml_sem_assinatura += ''.join(partes[-1])
            xml = xml_sem_assinatura

        # Colocamos o texto no avaliador XML
        doc_xml = etree.fromstring(xml.encode('utf-8'))
        signature_algorithm = 'rsa-sha1'
        digest_algorithm = 'sha1'
        if metodo == 'sha256':
            signature_algorithm = 'rsa-sha256'
            digest_algorithm = 'sha256'

        assinador = signxml.XMLSigner(
            method=signxml.methods.enveloped,
            signature_algorithm=signature_algorithm,
            digest_algorithm=digest_algorithm,
            c14n_algorithm='http://www.w3.org/TR/2001/REC-xml-c14n-20010315'
        )
        assinador.namespaces = {None: assinador.namespaces['ds']}
        doc_xml = assinador.sign(
            doc_xml, key=self.chave, cert=self.certificado, reference_uri=assinatura.URI)

        # Retransforma o documento xml em texto
        xml = etree.tounicode(doc_xml)
        xml = self._finaliza_xml(xml)
        return xml

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

==> PyNFe-5.0.11.tar.gz: PyNFe-5.0.11/pynfe/xml_sped/certificado.py <==
# -*- coding: utf-8 -*-

from pynfe.xml_sped.base import XMLNFe, ABERTURA
from pynfe.xml_sped.assinatura import Signature
from datetime import datetime
from OpenSSL import crypto
from lxml import etree
import signxml
import os


DIRNAME = os.path.dirname(__file__)


class Certificado(object):
    def __init__(self):
        self.arquivo = ''
        self.senha = ''
        self.chave = ''
        self.certificado = ''
        self.emissor = {}
        self.proprietario = {}
        self.data_inicio_validade = None
        self.data_fim_validade = None
        self._doc_xml = None
        self.cert_str=''
        self.key_str=''


    def _set_chave(self, chave):
        self._chave = chave

    def _get_chave(self):
        try:
            if self._chave:
                return self._chave
            else:
                raise AttributeError("'chave' precisa ser regenerada")
        except AttributeError as e:
            if self.arquivo:  # arquivo disponível
                self.prepara_certificado_arquivo_pfx()
                return self._chave  # agora já disponível
            else:
                return ''

    chave = property(_get_chave, _set_chave)

    def _set_certificado(self, certificado):
        self._certificado = certificado

    def _get_certificado(self):
        try:
            if self._certificado:
                return self._certificado
            else:
                raise AttributeError("'certificado' precisa ser regenerado")
        except AttributeError as e:
            if self.arquivo:  # arquivo disponível
                self.prepara_certificado_arquivo_pfx()
                return self._certificado  # agora já disponível
            else:
                return ''

    certificado = property(_get_certificado, _set_certificado)

    @property
    def proprietario_nome(self):
        if 'CN' in self.proprietario:
            #
            # Alguns certrificados não têm o CNPJ na propriedade CN, somente o
            # nome do proprietário
            #
            if ':' in self.proprietario['CN']:
                return self.proprietario['CN'].rsplit(':', 1)[0]
            else:
                return self.proprietario['CN']
        else: # chave CN ainda não disponível
            try:
                self.prepara_certificado_arquivo_pfx()
                return self.proprietario['CN'].rsplit(':', 1)[0]
            except IOError as e:  # arquivo do certificado não disponível
                return ''

    @property
    def proprietario_cnpj(self):
        if 'CN' in self.proprietario:
            #
            # Alguns certrificados não têm o CNPJ na propriedade CN, somente o
            # nome do proprietário
            #
            if ':' in self.proprietario['CN']:
                return self.proprietario['CN'].rsplit(':', 1)[1]
            else:
                return ''
        else:  # chave CN ainda não disponível
            try:
                self.prepara_certificado_arquivo_pfx()
                return self.proprietario['CN'].rsplit(':', 1)[1]
            except IOError as e:  # arquivo do certificado não disponível
                return ''

    def prepara_certificado_arquivo_pfx(self):

        self.chave=self.key_str
        self.prepara_certificado_txt(self.cert_str)

    def prepara_certificado_arquivo_pem(self):
        self.prepara_certificado_txt(self.cert_str)

    def prepara_certificado_txt(self, cert_txt):
        #
        # Para dar certo a leitura pelo xmlsec, temos que separar o certificado
        # em linhas de 64 caracteres de extensão...
        #
        cert_txt = cert_txt.replace('\n', '')
        cert_txt = cert_txt.replace('-----BEGIN CERTIFICATE-----', '')
        cert_txt = cert_txt.replace('-----END CERTIFICATE-----', '')
        linhas_certificado = ['-----BEGIN CERTIFICATE-----\n']
        for i in range(0, len(cert_txt), 64):
            linhas_certificado.append(cert_txt[i:i+64] + '\n')
        linhas_certificado.append('-----END CERTIFICATE-----\n')
        self.certificado = ''.join(linhas_certificado)
        cert_openssl = crypto.load_certificate(crypto.FILETYPE_PEM, self.certificado)
        self.emissor = dict(cert_openssl.get_issuer().get_components())
        self.proprietario = dict(cert_openssl.get_subject().get_components())
        self.data_inicio_validade = datetime.strptime(
            cert_openssl.get_notBefore().decode('utf-8'), '%Y%m%d%H%M%SZ')
        self.data_fim_validade = datetime.strptime(
            cert_openssl.get_notAfter().decode('utf-8'), '%Y%m%d%H%M%SZ')

    def assina_xmlnfe(self, doc):
        if not isinstance(doc, XMLNFe):
            raise ValueError('O documento nao e do tipo esperado: XMLNFe')

        # Realiza a assinatura
        xml = self.assina_xml(doc.xml)

        # Devolve os valores para a instância doc
        doc.Signature.xml = str(xml)

    def assina_arquivo(self, doc):
        xml = open(doc, 'r').read()
        xml = self.assina_xml(xml)
        return xml

    def _prepara_doc_xml(self, xml):
        #
        # Determina o tipo de arquivo que vai ser assinado, procurando
        # pela tag correspondente
        #
        if 'infNFe' in xml:
            doctype = '<!DOCTYPE NFe [<!ATTLIST infNFe Id ID #IMPLIED>]>'
        elif 'infCanc' in xml:
            doctype = '<!DOCTYPE cancNFe [<!ATTLIST infCanc Id ID #IMPLIED>]>'
        elif 'infEvento' in xml:
            doctype = '<!DOCTYPE envEvento [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        elif 'infInut' in xml:
            doctype = '<!DOCTYPE inutNFe [<!ATTLIST infInut Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        #
        # Importantíssimo colocar o encode, pois do contário não é possível
        # assinar caso o xml tenha letras acentuadas
        #
        xml = xml.replace(ABERTURA, '')
        xml = ABERTURA + xml
        xml = xml.replace(ABERTURA, ABERTURA + doctype)

        #
        # Remove todos os \n
        #
        xml = xml.replace('\n', '')

        return xml

    def _finaliza_xml(self, xml):
        #
        # Determina o tipo de arquivo que vai ser assinado, procurando
        # pela tag correspondente
        #
        if 'infNFe' in xml:
            doctype = '<!DOCTYPE NFe [<!ATTLIST infNFe Id ID #IMPLIED>]>'
        elif 'infCanc' in xml:
            doctype = '<!DOCTYPE cancNFe [<!ATTLIST infCanc Id ID #IMPLIED>]>'
        elif 'infEvento' in xml:
            doctype = '<!DOCTYPE envEvento [<!ATTLIST infEvento Id ID #IMPLIED>]>'
        elif 'infInut' in xml:
            doctype = '<!DOCTYPE inutNFe [<!ATTLIST infInut Id ID #IMPLIED>]>'
        else:
            raise ValueError('Tipo de arquivo desconhecido para assinatura/validacao')

        #
        # Remove o doctype e os \n acrescentados pela libxml2
        #
        xml = xml.replace('\n', '')
        xml = xml.replace(ABERTURA + doctype, ABERTURA)

        return xml

    def assina_xml(self, xml, metodo='sha1'):
        xml = self._prepara_doc_xml(xml)
        assinatura = Signature()
        assinatura.xml = str(xml)

        # Tiramos a tag de assinatura, o signxml vai colocar de novo
        if '<Signature ' in xml:
            partes = xml.split('<Signature ')
            xml_sem_assinatura = ''.join(partes[:-1])
            partes = partes[-1].split('</Signature>')
            xml_sem_assinatura += ''.join(partes[-1])
            xml = xml_sem_assinatura

        # Colocamos o texto no avaliador XML
        doc_xml = etree.fromstring(xml.encode('utf-8'))
        signature_algorithm = 'rsa-sha1'
        digest_algorithm = 'sha1'
        if metodo == 'sha256':
            signature_algorithm = 'rsa-sha256'
            digest_algorithm = 'sha256'

        assinador = signxml.XMLSigner(
            method=signxml.methods.enveloped,
            signature_algorithm=signature_algorithm,
            digest_algorithm=digest_algorithm,
            c14n_algorithm='http://www.w3.org/TR/2001/REC-xml-c14n-20010315'
        )
        assinador.namespaces = {None: assinador.namespaces['ds']}
        doc_xml = assinador.sign(
            doc_xml, key=self.chave, cert=self.certificado, reference_uri=assinatura.URI)

        # Retransforma o documento xml em texto
        xml = etree.tounicode(doc_xml)
        xml = self._finaliza_xml(xml)
        return xml

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
